/**
 * @file main.c
 * @brief Main routine
 *
 * @section License
 *
 * Copyright (C) 2021-2022 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneBOOT Eval.
 *
 * This software is provided in source form for a short-term evaluation only. The
 * evaluation license expires 90 days after the date you first download the software.
 *
 * If you plan to use this software in a commercial product, you are required to
 * purchase a commercial license from Oryx Embedded SARL.
 *
 * After the 90-day evaluation period, you agree to either purchase a commercial
 * license or delete all copies of this software. If you wish to extend the
 * evaluation period, you must contact sales@oryx-embedded.com.
 *
 * This evaluation software is provided "as is" without warranty of any kind.
 * Technical support is available as an option during the evaluation period.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 2.1.0
 **/

//Dependencies
#include <stdlib.h>
#include "stm32l4xx.h"
#include "stm32l4xx_hal.h"
#include "stm32l4xx_nucleo_144.h"
#include "resource_manager.h"
#if (IAP_FALLBACK_SUPPORT == ENABLED)
#include "iap/iap_fallback.h"
#endif
#include "iap_user_settings.h"
#include "uart_user.h"
#include "main.h"
#include "debug.h"

//Test macros
//#define TEST_MODE

#if defined(TEST_MODE)
//#define TEST_FLASH_R_W //uncomment if flash read write test or leave it commented to test flash swap banks
#define TEST_LED 0 // 0 green led, 1 blue led
#endif

//Global variables
const uint8_t* pemIapSignPublicKey;
size_t pemIapSignPublicKeyLen;

//IAP mutex handler
OsMutex iapMutex;

#if (IAP_FALLBACK_SUPPORT == ENABLED)
//User fallback routines
void fallbackUserTriggerInit(void);
error_t fallbackUserRoutine(void);
#endif


/**
 * @brief System clock configuration
 **/

void SystemClock_Config(void)
{
   RCC_OscInitTypeDef RCC_OscInitStruct = {0};
   RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
   RCC_PeriphCLKInitTypeDef RCC_PeriphClkInitStruct = {0};

   //Enable the LSE Oscilator
   RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI48;
   RCC_OscInitStruct.HSI48State = RCC_HSI48_ON;
   RCC_OscInitStruct.PLL.PLLState = RCC_PLL_OFF;
   HAL_RCC_OscConfig(&RCC_OscInitStruct);

   //Enable MSI Oscillator and activate PLL with MSI as source
   RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_MSI;
   RCC_OscInitStruct.MSIState = RCC_MSI_ON;
   RCC_OscInitStruct.MSIClockRange = RCC_MSIRANGE_7;
   RCC_OscInitStruct.MSICalibrationValue = RCC_MSICALIBRATION_DEFAULT;
   RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
   RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_MSI;
   RCC_OscInitStruct.PLL.PLLM = 1;
   RCC_OscInitStruct.PLL.PLLN = 20;
   RCC_OscInitStruct.PLL.PLLR = 2;
   RCC_OscInitStruct.PLL.PLLP = 7;
   RCC_OscInitStruct.PLL.PLLQ = 4;
   HAL_RCC_OscConfig(&RCC_OscInitStruct);

   //Enable MSI Auto-calibration through LSE
   HAL_RCCEx_EnableMSIPLLMode();

   //Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2
   //clocks dividers
   RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK |
      RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
   RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
   RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
   RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
   RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;
   HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_4);

   //Select clock source for RNG peripheral
   RCC_PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_RNG;
   RCC_PeriphClkInitStruct.RngClockSelection = RCC_RNGCLKSOURCE_HSI48;
   HAL_RCCEx_PeriphCLKConfig(&RCC_PeriphClkInitStruct);

   //Select clock source for USB peripheral
   RCC_PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_USB;
   RCC_PeriphClkInitStruct.UsbClockSelection = RCC_USBCLKSOURCE_HSI48;
   HAL_RCCEx_PeriphCLKConfig(&RCC_PeriphClkInitStruct);
}


/**
 * @brief LED task
 * @param[in] param Unused parameter
 **/

void ledTask(void *param)
{
   Led_TypeDef LEDx;

   //Select LED
#if defined(APP_DEMO_GREEN)
   LEDx = LED_GREEN;
#elif defined (APP_DEMO_YELLOW)
   LEDx = LED_BLUE;
#else
   LEDx = LED_RED;
#endif

   //Endless loop
   while(1)
   {
      BSP_LED_On(LEDx);
      osDelayTask(100);
      BSP_LED_Off(LEDx);
      osDelayTask(900);
   }
}


/**
 * @brief Load user RSA public key from "res.c".
 * "res.c" is a C array file system representing the demo folder "resources/keys".
 **/

error_t iapUserLoadKeys(void)
{
   error_t error;

   //Get server's private key
   error = resGetData(IAP_PUBLIC_SIGN_KEY, (const uint8_t **) &pemIapSignPublicKey,
      &pemIapSignPublicKeyLen);
   //Any error to report?
   if(error)
      return error;

   //Successful process
   return NO_ERROR;
}


#if (IAP_FALLBACK_SUPPORT == ENABLED)
/**
 * @brief Initialize the User fallback trigger
 * @return None
 **/

void fallbackUserTriggerInit(void)
{
   //Setup User button (B1 : PC13)

   // Enable the clock for GPIOC
   RCC->AHB2ENR |= (1 << 2);
   // Set PC13 as input
   GPIOC->MODER &= ~(1<<(13*2));
   GPIOC->MODER &= ~(1<<(13*2+1));

   // Set PC13 as Pull-Down
   GPIOC->PUPDR |= (2<<(13*2));
}


/**
 * @brief Fallback User routine. It executes the CycloneBOOT IAP
 * fallback according the defined User trigger action.
 * @return Status code
 **/

error_t fallbackUserRoutine(void)
{
   error_t error;

   //Initialize status code
   error = NO_ERROR;

   //Initialize the fallback trigger (here User Button -> B1 : PC13)
   fallbackUserTriggerInit();

   //Is fallback trigger hit?
   if ((GPIOC->IDR >> 13) & 0x1)
   {
      //wait button release
      while ((GPIOC->IDR >> 13) & 0x1);

      //Start CycloneBOOT IAP Fallback procedure
      error = iapFallbackStart();
   }

   //Return status code
   return error;
}
#endif


#if defined(TEST_MODE)
#include "drivers/flash/internal/stm32l4xx_flash_driver.h"

error_t testRoutine(void)
{
   uint_t testIndex;
   uint8_t dataR[128];
   uint8_t dataW[128];
   size_t writeSize;
   uint32_t eraseStartAddr;
   uint32_t addrTest;
   size_t eraseLength;
   error_t error;
   FlashDriver *flashDrv = (FlashDriver*) &stm32l4xxFlashDriver;
   const FlashInfo *flashInfo;
   uint_t i;

   error = NO_ERROR;
   testIndex = 1;
   addrTest = STM32L4xx_FLASH_BANK2_ADDR;

   //Init flash driver
   flashDrv->init();

   error = flashDrv->getInfo(&flashInfo);
   if(error)
   {
      TRACE_ERROR("Failed to read flash!\r\n");
      return error;
   }

#if 1
   TRACE_INFO("\r\n");
   //Stuff in bank2
   if(READ_BIT(FLASH->OPTR, FLASH_OPTR_BFB2))
   {
      TRACE_INFO("Running on bank2!\r\n");
   }
   //Stuff in bank1
   else
   {
      TRACE_INFO("Running on bank1!\r\n");
   }
   TRACE_INFO("\r\n");

   TRACE_INFO("Push user button to swap banks 0\r\n");
   while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
   while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

   do
   {
      ///////////////////////////////////////////////////////////
      addrTest = 0x08080000;
      memset(dataW, 0x11, 32);
      memset(dataW+32, 0x22, 32);
      memset(dataW+64, 0x33, 32);
      memset(dataW+96, 0x44, 32);

      TRACE_INFO("Writing data (%d bytes) at 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->write(addrTest, dataW, sizeof(dataW));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      memset(dataR, 0xFF, sizeof(dataR));
      TRACE_INFO("Reading data (%d bytes) from 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->read(addrTest, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Compare written/read data (%d bytes) to/from 0x%08X...\r\n", sizeof(dataW), addrTest);
      if(memcmp(dataW, dataR, sizeof(dataW)) != 0)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Push user button to swap banks 1.1\r\n");
      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

      TRACE_INFO("Erasing data (%d bytes) at 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->erase(addrTest, sizeof(dataW));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Push user button to swap banks 1.2\r\n");
      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

      memset(dataR, 0x00, sizeof(dataR));
      memset(dataW, 0xFF, sizeof(dataR));

      TRACE_INFO("Reading data (%d bytes) from 0x%08X after erase...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->read(addrTest, dataR, sizeof(dataR));
      if(error)
         break;

      TRACE_INFO("Compare erased/read data (%d bytes) to/from 0x%08X...\r\n", sizeof(dataW), addrTest);
      if(memcmp(dataW, dataR, sizeof(dataW)) != 0)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      memset(dataW, 0xAA, sizeof(dataW));

      TRACE_INFO("Writing data (%d bytes) at 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->write(addrTest, dataW, sizeof(dataW));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      memset(dataR, 0xFF, sizeof(dataR));
      TRACE_INFO("Reading data (%d bytes) from 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->read(addrTest, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Compare written/read data (%d bytes) to/from 0x%08X...\r\n", sizeof(dataW), addrTest);
      if(memcmp(dataW, dataR, sizeof(dataW)) != 0)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Push user button to swap banks 1.3\r\n");
      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

      ///////////////////////////////////////////////////////////
      ///////////////////////////////////////////////////////////
      ///////////////////////////////////////////////////////////

      TRACE_INFO("\r\n");
      TRACE_INFO("\r\n");
      TRACE_INFO("\r\n");


      addrTest = 0x08000000 + (0x80000 - 0x800);
      memset(dataW, 0x55, 32);
      memset(dataW+32, 0x66, 32);
      memset(dataW+64, 0x77, 32);
      memset(dataW+96, 0x88, 32);

      TRACE_INFO("Writing data (%d bytes) at 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->write(addrTest, dataW, sizeof(dataW));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      memset(dataR, 0xFF, sizeof(dataR));
      TRACE_INFO("Reading data (%d bytes) from 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->read(addrTest, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Compare written/read data (%d bytes) to/from 0x%08X...\r\n", sizeof(dataW), addrTest);
      if(memcmp(dataW, dataR, sizeof(dataW)) != 0)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Push user button to swap banks 2.1\r\n");
      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

      TRACE_INFO("Erasing data (%d bytes) at 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->erase(addrTest, sizeof(dataW));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Push user button to swap banks 2.2\r\n");
      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

      memset(dataR, 0x00, sizeof(dataR));
      memset(dataW, 0xFF, sizeof(dataR));

      TRACE_INFO("Reading data (%d bytes) from 0x%08X after erase...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->read(addrTest, dataR, sizeof(dataR));
      if(error)
         break;

      TRACE_INFO("Compare erased/read data (%d bytes) to/from 0x%08X...\r\n", sizeof(dataW), addrTest);
      if(memcmp(dataW, dataR, sizeof(dataW)) != 0)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      memset(dataW, 0xBB, sizeof(dataW));

      TRACE_INFO("Writing data (%d bytes) at 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->write(addrTest, dataW, sizeof(dataW));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      memset(dataR, 0xFF, sizeof(dataR));
      TRACE_INFO("Reading data (%d bytes) from 0x%08X...\r\n", sizeof(dataW), addrTest);
      error = flashDrv->read(addrTest, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Compare written/read data (%d bytes) to/from 0x%08X...\r\n", sizeof(dataW), addrTest);
      if(memcmp(dataW, dataR, sizeof(dataW)) != 0)
      {
         TRACE_INFO("NOK!\r\n");
         break;
      }
      TRACE_INFO("OK!\r\n");

      TRACE_INFO("Push user button to swap banks 2.3\r\n");
      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);

   }while(0);

   TRACE_INFO("Done!\r\n");
   __NOP();

#elif defined(TEST_FLASH_R_W)

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////


   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Read 1 byte\r\n", testIndex);

   do {
      error = NO_ERROR;
      memset(dataR, 0, sizeof(dataR));

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(uint8_t));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, sizeof(uint8_t));
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Read 1 word\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataR, 0, sizeof(dataR));

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(uint32_t));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, sizeof(uint32_t));
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Read 128 bytes\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataR, 0, sizeof(dataR));

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, sizeof(dataR));
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n", testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Write 1 byte\r\n", testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xAA, sizeof(uint8_t));

      writeSize = (sizeof(uint8_t)*flashInfo->writeSize)/flashInfo->writeSize + ((sizeof(uint8_t) % flashInfo->writeSize) > 0);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      error = flashDrv->write(STM32L4xx_FLASH_BANK2_ADDR, dataW, sizeof(uint8_t));
      if(error)
      {
         TRACE_ERROR("Failed to write flash!\r\n");
         break;
      }

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

      if(memcmp(dataW, dataR, writeSize) != 0)
      {
         error = ERROR_FAILURE;
         break;
      }
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Write 1 word\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xBB, sizeof(uint32_t));

      writeSize = (sizeof(uint32_t)*flashInfo->writeSize)/flashInfo->writeSize + ((sizeof(uint32_t) % flashInfo->writeSize) > 0);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      error = flashDrv->write(STM32L4xx_FLASH_BANK2_ADDR, dataW, sizeof(uint32_t));
      if(error)
      {
         TRACE_ERROR("Failed to write flash!\r\n");
         break;
      }

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

      if(memcmp(dataW, dataR, writeSize) != 0)
      {
         error = ERROR_FAILURE;
         break;
      }
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Write 8 words\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xCC, flashInfo->writeSize);

      writeSize = (flashInfo->writeSize*flashInfo->writeSize)/flashInfo->writeSize + ((flashInfo->writeSize % flashInfo->writeSize) > 0);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      error = flashDrv->write(STM32L4xx_FLASH_BANK2_ADDR, dataW, flashInfo->writeSize);
      if(error)
      {
         TRACE_ERROR("Failed to write flash!\r\n");
         break;
      }

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

      if(memcmp(dataW, dataR, writeSize) != 0)
      {
         error = ERROR_FAILURE;
         break;
      }
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Write 128 words\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xDD, sizeof(dataW));

      writeSize = (sizeof(dataW)*flashInfo->writeSize)/flashInfo->writeSize + ((sizeof(dataW) % flashInfo->writeSize) > 0);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      error = flashDrv->write(STM32L4xx_FLASH_BANK2_ADDR, dataW, sizeof(dataW));
      if(error)
      {
         TRACE_ERROR("Failed to write flash!\r\n");
         break;
      }

      //Read first bank 2 sector 128bytes
      error = flashDrv->read(STM32L4xx_FLASH_BANK2_ADDR, dataR, sizeof(dataR));
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

      if(memcmp(dataW, dataR, writeSize) != 0)
      {
         error = ERROR_FAILURE;
         break;
      }
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Erase flash bank 1 last sector\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xEE, sizeof(dataW));

      eraseStartAddr = STM32L4xx_FLASH_BANK1_ADDR+STM32L4xx_FLASH_SECTOR_SIZE*(STM32L4xx_FLASH_SECTOR_NUMBER/2-1);
      eraseLength = STM32L4xx_FLASH_SECTOR_SIZE;

      writeSize = sizeof(dataW);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      error = flashDrv->write(eraseStartAddr, dataW, writeSize);
      if(error)
      {
         TRACE_ERROR("Failed to write flash!\r\n");
         break;
      }

      error = flashDrv->erase(eraseStartAddr, eraseLength);
      if(error)
      {
         TRACE_ERROR("Failed to erase flash sector(s)!\r\n");
         break;
      }


      //Read first bank 2 sector 128bytes
      error = flashDrv->read(eraseStartAddr, dataR, writeSize);
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

      memset(dataW, 0xFF, writeSize);
      if(memcmp(dataW, dataR, writeSize) != 0)
      {
         error = ERROR_FAILURE;
         break;
      }
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Erase flash bank 2 last sector\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xEE, sizeof(dataW));

      eraseStartAddr = STM32L4xx_FLASH_BANK2_ADDR+STM32L4xx_FLASH_SECTOR_SIZE*(STM32L4xx_FLASH_SECTOR_NUMBER/2-1);
      eraseLength = STM32L4xx_FLASH_SECTOR_SIZE;

      writeSize = sizeof(dataW);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      error = flashDrv->write(eraseStartAddr, dataW, writeSize);
      if(error)
      {
         TRACE_ERROR("Failed to write flash!\r\n");
         break;
      }

      error = flashDrv->erase(eraseStartAddr, eraseLength);
      if(error)
      {
         TRACE_ERROR("Failed to erase flash sector(s)!\r\n");
         break;
      }


      //Read first bank 2 sector 128bytes
      error = flashDrv->read(eraseStartAddr, dataR, writeSize);
      if(error)
      {
         TRACE_ERROR("Failed to read flash!\r\n");
         break;
      }

      TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

      memset(dataW, 0xFF, writeSize);
      if(memcmp(dataW, dataR, writeSize) != 0)
      {
         error = ERROR_FAILURE;
         break;
      }
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Erase 3 sector in flash bank 2\r\n",testIndex);

   do {
      error = NO_ERROR;
      memset(dataW, 0, sizeof(dataW));
      memset(dataW, 0xEE, sizeof(dataW));

      eraseStartAddr = STM32L4xx_FLASH_BANK2_ADDR;
      eraseLength = STM32L4xx_FLASH_SECTOR_SIZE*3;

      writeSize = sizeof(dataW);

      TRACE_INFO_ARRAY("WRITE: ", dataW, writeSize);

      for(i = 0; i < 3; i++)
      {
         //write in sectors
         error = flashDrv->write(eraseStartAddr+i*STM32L4xx_FLASH_SECTOR_SIZE, dataW, writeSize);
         if(error)
         {
            TRACE_ERROR("Failed to write flash!\r\n");
            break;
         }
      }
      if(error)
         break;

      error = flashDrv->erase(eraseStartAddr, eraseLength);
      if(error)
      {
         TRACE_ERROR("Failed to erase flash sector(s)!\r\n");
         break;
      }

      //SCB_InvalidateICache();
      //SCB_InvalidateDCache();

      memset(dataW, 0xFF, writeSize);

      for(i = 0; i < 3; i++)
      {
         memset(dataR, 0x00, writeSize);
         //Read bank 1
         error = flashDrv->read(eraseStartAddr+i*STM32L4xx_FLASH_SECTOR_SIZE, dataR, writeSize);
         if(error)
         {
            TRACE_ERROR("Failed to read flash!\r\n");
            break;
         }

         TRACE_INFO_ARRAY("READ: ", dataR, writeSize);

         if(memcmp(dataW, dataR, writeSize) != 0)
         {
            error = ERROR_FAILURE;
            break;
         }
      }

   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;


   #else

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////


   TRACE_INFO("\r\n");
   TRACE_INFO("TEST%d: Swap flash banks\r\n",testIndex);

   do {

#if (defined(TEST_LED) && (TEST_LED == 0))
      BSP_LED_On(LED1);
      BSP_LED_Off(LED2);
#else
      BSP_LED_Off(LED1);
      BSP_LED_On(LED2);
#endif
      TRACE_INFO("Push user button to swap banks\r\n");

      while (BSP_PB_GetState(BUTTON_USER) != GPIO_PIN_SET);
      while (BSP_PB_GetState(BUTTON_USER) == GPIO_PIN_SET);


      error = flashDrv->swapBanks();
      if(error)
      {
         TRACE_ERROR("Failed to swap banks!\r\n");
         break;
      }

      NVIC_SystemReset();
   }while(0);

   if(error)
      TRACE_INFO("TEST%d: Failed!\r\n",testIndex);
   else
      TRACE_INFO("TEST%d: OK\r\n",testIndex);

   testIndex++;
#endif

   return 0;
}
#endif


/**
 * @brief Main entry point
 * @return Unused value
 **/

int_t main(void)
{
#if defined(IAP_SCENARIO_3)
   error_t error;
#endif
   OsTaskId taskId;

#if (IAP_FALLBACK_SUPPORT == ENABLED)
   //Call user fallback routine
   fallbackUserRoutine();
#endif

   //HAL library initialization
   HAL_Init();
   //Configure the system clock
   SystemClock_Config();

   //Initialize kernel
   osInitKernel();
   //Configure debug UART
   debugInit(115200);

   //Start-up message
   TRACE_INFO("\r\n");
   TRACE_INFO("***********************************************\r\n");
   TRACE_INFO("*** CycloneBOOT UART Y-MODEM Dual Bank Demo ***\r\n");
   TRACE_INFO("***********************************************\r\n");
#if defined(IAP_SCENARIO_1)
   TRACE_INFO("IAP mode: Simple APP + Integrity\r\n");
#elif defined (IAP_SCENARIO_2)
   TRACE_INFO("IAP mode: Cipher APP + Authentification\r\n");
#elif defined(IAP_SCENARIO_3)
   TRACE_INFO("IAP mode: Cipher APP + Signature\r\n");
#else
#endif
#if defined(APP_DEMO_GREEN)
   TRACE_INFO("Version: 2.0.0 (GREEN)\r\n");
#elif defined(APP_DEMO_BLUE)
   TRACE_INFO("Version: 3.0.0 (BLUE)\r\n");
#else
   TRACE_INFO("Version: 1.0.0 (RED)\r\n");
#endif
   TRACE_INFO("Copyright: 2010-2022 Oryx Embedded SARL\r\n");
   TRACE_INFO("Compiled: %s %s\r\n", __DATE__, __TIME__);
   TRACE_INFO("Target: STM32L496\r\n");
   TRACE_INFO("\r\n");

   //LED configuration
   BSP_LED_Init(LED1);
   BSP_LED_Init(LED2);
   BSP_LED_Init(LED3);

   //Clear LEDs
   BSP_LED_Off(LED1);
   BSP_LED_Off(LED2);
   BSP_LED_Off(LED3);

   //Initialize user button
   BSP_PB_Init(BUTTON_KEY, BUTTON_MODE_GPIO);

#if defined(TEST_MODE)
   testRoutine();
   return 0;
#endif

#if defined (IAP_SCENARIO_3)
   //Load IAP public signature key
   error = iapUserLoadKeys();
   if (error)
   {
	  //Debug message
      TRACE_ERROR("Failed to load IAP user keys!\r\n");
      return error;
   }
#endif

   //Create IAP mutex
   if(!osCreateMutex(&iapMutex))
   {
      //Debug message
      TRACE_ERROR("Failed to create IAP mutex!\r\n");
   }

   //Create user task
   taskId = osCreateTask("UART User", uartUserTask, NULL, 500, OS_TASK_PRIORITY_NORMAL);
   //Failed to create the task?
   if(taskId == OS_INVALID_TASK_ID)
   {
      //Debug message
      TRACE_ERROR("Failed to create task!\r\n");
   }

   //Create a task to blink the LED
   taskId = osCreateTask("LED", ledTask, NULL, 200, OS_TASK_PRIORITY_NORMAL);
   //Failed to create the task?
   if(taskId == OS_INVALID_TASK_ID)
   {
      //Debug message
      TRACE_ERROR("Failed to create task!\r\n");
   }

   //Start the execution of tasks
   osStartKernel();

   //This function should never return
   return 0;
}

/**
 * @brief Freertos User task stack overflow callback
 * @param[in] xTask Task handler of the task having a stack overflow
 * @param[in] pcTaskName Task name of the task having a stack overflow
 **/

void vApplicationStackOverflowHook(TaskHandle_t xTask,
                                       char *pcTaskName)
{
   TRACE_ERROR("Stack overflow in \"%s\"!\r\n", pcTaskName);
   vTaskSuspendAll();
   while(1)
   {
   }
}


/**
 * @brief Freertos User malloc failed callback
 **/

void vApplicationMallocFailedHook(void)
{
   TRACE_ERROR("Malloc failed!\r\n");
   vTaskSuspendAll();
   while(1)
   {
   }
}
